#---------------------------------------------------------------------------
# Robô com RaspberryPi Pico
#
# Arquivo: robo_raspPico.py
# Versão : 1.0
# Plataforma: Raspberry Pi Pico - Thonny
#
# Desenvolvido por: Eng. Márcio José Soares
#
# Pinos utilizados:
# GPIO02 - motor In4 - pino 4
# GPIO03 - motor In3 - pino 5
# GPIO04 - motor In2 - pino 6
# GPIO05 - motor In1 - pino 7
# GPIO06 - motor ENB - pino 9 (PWM) 
# GPIO07 - motor ENA - pino 10 (PWM)
# GPIO13 - Buzer     - pino 17
# GPIO22 - LED Live2 - pino 29
# GPIO25 - LED Live1 - LED Builtin
#
#---------------------------------------------------------------------------
#
# Últimas alterações:
# em 26/11/2021:
#   - inicio do projeto
# em 31/01/2022:
#   - inserido LED Live e testado programa com Thonny no desktop novo!
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
# Bibliotecas incluídas
#---------------------------------------------------------------------------
from machine import Pin
import time
import utime

#---------------------------------------------------------------------------
# Define pinos
#---------------------------------------------------------------------------
pinIni1Esq = 2                                        #
pinIni2Esq = 3
pinIni3Dir = 4
pinIni4Dir = 5

pinTrig = 14
pinEcho = 15
pinBuzzer = 13

pinLED_Live1 = 25
pinLED_Live2 = 22

#---------------------------------------------------------------------------
# Configura pinos
#---------------------------------------------------------------------------
In1Esq = Pin(pinIni1Esq, Pin.OUT)                     # cria pinos de controle da ponte H
In2Esq = Pin(pinIni2Esq, Pin.OUT)   
                                         
In3Dir = Pin(pinIni3Dir, Pin.OUT)         
In4Dir = Pin(pinIni4Dir, Pin.OUT)
                     
EnEsq = machine.PWM(machine.Pin(6))                   # pinos com PWM
EnDir = machine.PWM(machine.Pin(7))

EchoS   = Pin(pinEcho, Pin.IN)                        # pinos para uso no sonar
TrigerS = Pin(pinTrig, Pin.OUT)

LedLive1 = Pin(pinLED_Live1, Pin.OUT)
LedLive2 = Pin(pinLED_Live2, Pin.OUT)

Buzzer  = machine.PWM(machine.Pin(pinBuzzer))         # pino do buzzer

EnEsq.freq(1000);                                     # frequencia nos pinos de PWM                      
EnDir.freq(1000);                         
Buzzer.freq(1000);

#---------------------------------------------------------------------------
# Vars globais
#---------------------------------------------------------------------------
duty = 20000 #//23000                                          # Duty cicle inicial
duty_05 = 18000#21000

#---------------------------------------------------------------------------
# Função sonar
#
# Adaptada do exemplo obtido em Brincando com ideias
#
# Entradas - nenhuma
# Saídas   - distância medida
#---------------------------------------------------------------------------
def sonar():
    
    new = False
    counter = 0
    result = 0
    dur = 0
    
    TrigerS.value(0)                                  # Pino em Low
    time.sleep_us(2)                                  # aguarda 
    TrigerS.value(1)                                  # levanta pino
    time.sleep_us(10)                                 # aguarda
    TrigerS.value(0)                                  # pino em low
    time.sleep_us(2)                                  # aguarda
    
    while not EchoS.value():                          # se pino em low
       pass
       counter += 1                                   # incrementa contador
       if counter == 5000:                            # se chegou no limite
           new = True                                 # prepara retorno
           break                                      # sai do laço
        
    if new:                                           # retorna falso
       return False
    
    inicio = time.ticks_us()/1000000                  # pega tempo
    
    while EchoS.value():                              # lê sinal de retorno
        pass                                          # sai quando em low
    
    ret = time.ticks_us()/1000000                     # lê novo tempo
    
    if ret == inicio:                                 # se tempo igual, retorna
        result = "N/A"
    else:
        dur = ret - inicio                            # do contrário, calcula distância
        
    soundSpeed = 34300                                # velocidade do som
    result = dur * soundSpeed / 2                     # só interessa tempo de ida
    result = round(result, 1)                         # uma casa decimal, arredonda se necessário
    return result                                     # retorna a distância medida

#---------------------------------------------------------------------------
# Função stopMotors
#
# Entradas - nenhuma
# Saídas   - nenhuma
#---------------------------------------------------------------------------                
def stopMotors():
    
    print("pare o robô")                              # mensagem na tela
    In1Esq.low()                                      # acerta pinos
    In2Esq.low()                   
    In3Dir.low()             
    In4Dir.low()
    EnEsq.duty_u16(0)                                 # duty em zero!
    EnDir.duty_u16(0)                   
            
#---------------------------------------------------------------------------
# Função para frente
#
# Entradas - valor do duty cicle desejado
# Saídas   - nenhuma
#--------------------------------------------------------------------------- 
def goForward(myduty):
    
    print("p/ frente")                                # mensagem na tela
    In1Esq.high()                                     # acerta pinos
    In2Esq.low()                   
    In3Dir.low()             
    In4Dir.high()
    EnEsq.duty_u16(myduty)                            # liga PWM
    EnDir.duty_u16(myduty)
       
#---------------------------------------------------------------------------
# Função para trás
#
# Entradas - valor do duty cicle desejado
# Saídas   - nenhuma
#--------------------------------------------------------------------------- 
def goBack(myduty):
    
    print("p/ tras")                                  # mensagem na tela     
    In1Esq.low()                                      # acerta pinos
    In2Esq.high()                   
    In3Dir.high()             
    In4Dir.low()
    EnEsq.duty_u16(myduty)                            # liga PWM
    EnDir.duty_u16(myduty)
    
#---------------------------------------------------------------------------
# Função para direita
#
# Entradas - valor do duty cicle desejado
# Saídas   - nenhuma
#--------------------------------------------------------------------------- 
def goRight(myduty):
    
    print("p/ direita")                               # mensagem na tela     
    In1Esq.high()                                     # acerta pinos
    In2Esq.low()                   
    In3Dir.high()             
    In4Dir.low()
    EnEsq.duty_u16(myduty)                            # liga PWM
    EnDir.duty_u16(myduty)
    
#---------------------------------------------------------------------------
# Função para esquerda
#
# Entradas - valor do duty cicle desejado
# Saídas   - nenhuma
#--------------------------------------------------------------------------- 
def goLeft(myduty):
    
    print("p/ esquerda")                              # mensagem na tela   
    In1Esq.low()                                      # acerta pinos
    In2Esq.high()                   
    In3Dir.low()             
    In4Dir.high()
    EnEsq.duty_u16(myduty)                            # liga PWM
    EnDir.duty_u16(myduty)
    
    
#---------------------------------------------------------------------------
# Função "main" (não existe em Python, mas sou programador "old school" de C!!!)
#
# Entradas - nenhuma
# Saídas   - nenhuma
#--------------------------------------------------------------------------- 

mycount = 0
myLED = False

LedLive1.low()
LedLive2.low()

stopMotors()                                          # inicia com motores parados
Buzzer.duty_u16(0)                                    # buzzer desligado
utime.sleep(2)                                        # aguarda 2 segundos para iniciar
                                          
print("Inicio do andar do robô!!!")                   # mensagem inicial               
                                          
while True:                                           # laço eterno!
    dist = sonar()                                    # mede a distância
    print("Dist: "+str(dist))                         # print
    time.sleep(0.25)                                  # aguarda 0.25s
    mycount = mycount + 1                             # soma 1
    if mycount == 2:                                  # se ja passou 0.5s
        mycount = 0
        if myLED:                                     # verifica estado do LED e troca
            myLED = False                             # o estado 
            LedLive1.low()
            LedLive2.high()
        else:
            myLED = True
            LedLive1.high()
            LedLive2.low()
    #continue

    if dist >= 10.0:                                  # se a distância maior ou igual a 10cm
       goForward(duty)                                # robô a frente
    else:                                             # do contrário
       Buzzer.duty_u16(1000)                          # liga buzzer
       stopMotors()                                   # para os motores
       goBack(duty_05)                                # anda para trás
       time.sleep(1)                                  # por 1 segundo
       stopMotors()                                   # para
       goLeft(duty_05)                                # vira para esquerda
       time.sleep(1)                                  # por 1 segundo
       stopMotors()                                   # para
       Buzzer.duty_u16(0)                             # desliga buzzer e começa tudo novamente
       